@@ -38,6 +38,7 @@
 #include <string.h>
 #include <stdbool.h>
 #include <assert.h>
+#include <errno.h>
 
 #include <nbdkit-filter.h>
 
@@ -50,10 +51,14 @@ static enum FuaMode {
   DISCARD,
 } fuamode;
 
+static bool flush_on_close = false;
+
 static int
 fua_config (nbdkit_next_config *next, nbdkit_backend *nxdata,
             const char *key, const char *value)
 {
+  int r;
+
   if (strcmp (key, "fuamode") == 0) {
     if (strcmp (value, "none") == 0)
       fuamode = NONE;
@@ -73,17 +78,25 @@ fua_config (nbdkit_next_config *next, nbdkit_backend *nxdata,
     }
     return 0;
   }
+  else if (strcmp (key, "flush-on-close") == 0 ||
+           strcmp (key, "flush_on_close") == 0) {
+    r = nbdkit_parse_bool (value);
+    if (r == -1)
+      return -1;
+    flush_on_close = r;
+    return 0;
+  }
   return next (nxdata, key, value);
 }
 
 #define fua_config_help \
   "fuamode=<MODE>       One of 'none' (default), 'emulate', 'native',\n" \
-  "                       'force', 'pass'."
+  "                       'force', 'pass'.\n" \
+  "flush-on-close=true  Force flush on client close"
 
 /* Check that desired mode is supported by plugin. */
 static int
-fua_prepare (nbdkit_next *next, void *handle,
-             int readonly)
+fua_prepare (nbdkit_next *next, void *handle, int readonly)
 {
   int r;
 
@@ -117,6 +130,17 @@ fua_prepare (nbdkit_next *next, void *handle,
     }
     break;
   }
+
+  if (flush_on_close) {
+    r = next->can_flush (next);
+    if (r == -1)
+      return -1;
+    if (r == 0) {
+      nbdkit_error ("flush-on-close requires plugin flush support");
+      return -1;
+    }
+  }
+
   return 0;
 }
 
@@ -157,42 +181,7 @@ fua_can_fua (nbdkit_next *next, void *handle)
 }
 
 static int
-fua_pwrite (nbdkit_next *next,
-            void *handle, const void *buf, uint32_t count, uint64_t offs,
-            uint32_t flags, int *err)
-{
-  int r;
-  bool need_flush = false;
-
-  switch (fuamode) {
-  case NONE:
-    assert (!(flags & NBDKIT_FLAG_FUA));
-    break;
-  case EMULATE:
-    if (flags & NBDKIT_FLAG_FUA) {
-      need_flush = true;
-      flags &= ~NBDKIT_FLAG_FUA;
-    }
-    break;
-  case NATIVE:
-  case PASS:
-    break;
-  case FORCE:
-    flags |= NBDKIT_FLAG_FUA;
-    break;
-  case DISCARD:
-    flags &= ~NBDKIT_FLAG_FUA;
-    break;
-  }
-  r = next->pwrite (next, buf, count, offs, flags, err);
-  if (r != -1 && need_flush)
-    r = next->flush (next, 0, err);
-  return r;
-}
-
-static int
-fua_flush (nbdkit_next *next,
-           void *handle, uint32_t flags, int *err)
+fua_flush (nbdkit_next *next, void *handle, uint32_t flags, int *err)
 {
   switch (fuamode) {
   case FORCE:
@@ -208,6 +197,49 @@ fua_flush (nbdkit_next *next,
   abort ();
 }
 
+/* Common code to update the flags for write-like operations according
+ * to the fuamode.
+ */
+static void
+update_flags (uint32_t *flags, bool *need_flush)
+{
+  switch (fuamode) {
+  case NONE:
+    assert (!(*flags & NBDKIT_FLAG_FUA));
+    break;
+  case EMULATE:
+    if (*flags & NBDKIT_FLAG_FUA) {
+      *need_flush = true;
+      *flags &= ~NBDKIT_FLAG_FUA;
+    }
+    break;
+  case NATIVE:
+  case PASS:
+    break;
+  case FORCE:
+    *flags |= NBDKIT_FLAG_FUA;
+    break;
+  case DISCARD:
+    *flags &= ~NBDKIT_FLAG_FUA;
+    break;
+  }
+}
+
+static int
+fua_pwrite (nbdkit_next *next,
+            void *handle, const void *buf, uint32_t count, uint64_t offs,
+            uint32_t flags, int *err)
+{
+  int r;
+  bool need_flush = false;
+
+  update_flags (&flags, &need_flush);
+  r = next->pwrite (next, buf, count, offs, flags, err);
+  if (r != -1 && need_flush)
+    r = next->flush (next, 0, err);
+  return r;
+}
+
 static int
 fua_trim (nbdkit_next *next,
           void *handle, uint32_t count, uint64_t offs, uint32_t flags,
@@ -216,26 +248,7 @@ fua_trim (nbdkit_next *next,
   int r;
   bool need_flush = false;
 
-  switch (fuamode) {
-  case NONE:
-    assert (!(flags & NBDKIT_FLAG_FUA));
-    break;
-  case EMULATE:
-    if (flags & NBDKIT_FLAG_FUA) {
-      need_flush = true;
-      flags &= ~NBDKIT_FLAG_FUA;
-    }
-    break;
-  case NATIVE:
-  case PASS:
-    break;
-  case FORCE:
-    flags |= NBDKIT_FLAG_FUA;
-    break;
-  case DISCARD:
-    flags &= ~NBDKIT_FLAG_FUA;
-    break;
-  }
+  update_flags (&flags, &need_flush);
   r = next->trim (next, count, offs, flags, err);
   if (r != -1 && need_flush)
     r = next->flush (next, 0, err);
@@ -250,32 +263,30 @@ fua_zero (nbdkit_next *next,
   int r;
   bool need_flush = false;
 
-  switch (fuamode) {
-  case NONE:
-    assert (!(flags & NBDKIT_FLAG_FUA));
-    break;
-  case EMULATE:
-    if (flags & NBDKIT_FLAG_FUA) {
-      need_flush = true;
-      flags &= ~NBDKIT_FLAG_FUA;
-    }
-    break;
-  case NATIVE:
-  case PASS:
-    break;
-  case FORCE:
-    flags |= NBDKIT_FLAG_FUA;
-    break;
-  case DISCARD:
-    flags &= ~NBDKIT_FLAG_FUA;
-    break;
-  }
+  update_flags (&flags, &need_flush);
   r = next->zero (next, count, offs, flags, err);
   if (r != -1 && need_flush)
     r = next->flush (next, 0, err);
   return r;
 }
 
+static int
+fua_finalize (nbdkit_next *next, void *handle)
+{
+  int r, err;
+
+  if (flush_on_close) {
+    /* We checked in fua_prepare that this plugin supports flush. */
+    r = next->flush (next, 0, &err);
+    if (r == -1) {
+      errno = err;
+      return -1;
+    }
+  }
+
+  return 0;
+}
+
 static struct nbdkit_filter filter = {
   .name              = "fua",
   .longname          = "nbdkit fua filter",
@@ -288,6 +299,7 @@ static struct nbdkit_filter filter = {
   .flush             = fua_flush,
   .trim              = fua_trim,
   .zero              = fua_zero,
+  .finalize          = fua_finalize,
 };
 
 NBDKIT_REGISTER_FILTER (filter)
