@@ -71,6 +71,12 @@ static long protocols = CURLPROTO_ALL;
 static const char *protocols = NULL;
 #endif
 static const char *proxy = NULL;
+#ifdef HAVE_CURLOPT_PROXY_CAINFO
+static const char *proxy_cainfo = NULL;
+#endif
+#ifdef HAVE_CURLOPT_PROXY_CAPATH
+static const char *proxy_capath = NULL;
+#endif
 static char *proxy_password = NULL;
 static const char *proxy_user = NULL;
 static struct curl_slist *resolves = NULL;
@@ -360,6 +366,26 @@ curl_config (const char *key, const char *value)
     proxy = value;
   }
 
+  else if (strcmp (key, "proxy-cainfo") == 0) {
+#ifdef HAVE_CURLOPT_PROXY_CAINFO
+    proxy_cainfo = value;
+#else
+    nbdkit_error ("'%s' is not supported in this version of curl",
+                  "CURLOPT_PROXY_CAINFO");
+    return -1;
+#endif
+  }
+
+  else if (strcmp (key, "proxy-capath") == 0) {
+#ifdef HAVE_CURLOPT_PROXY_CAPATH
+    proxy_capath = value;
+#else
+    nbdkit_error ("'%s' is not supported in this version of curl",
+                  "CURLOPT_PROXY_CAPATH");
+    return -1;
+#endif
+  }
+
   else if (strcmp (key, "proxy-password") == 0) {
     free (proxy_password);
     if (nbdkit_read_password (value, &proxy_password) == -1)
@@ -534,6 +560,8 @@ const char *curl_config_help =
   "password=<PASSWORD>        The password for the user account.\n"
   "protocols=PROTO,PROTO,..   Limit protocols allowed.\n"
   "proxy=<PROXY>              Set proxy URL.\n"
+  "proxy-cainfo=<CAINFO>      Path to Certificate Authority file for proxy.\n"
+  "proxy-capath=<CAPATH>      Path to directory with CA certs for proxy.\n"
   "proxy-password=<PASSWORD>  The proxy password.\n"
   "proxy-user=<USER>          The proxy user.\n"
   "resolve=<HOST>:<PORT>:<ADDR> Custom host to IP address resolution.\n"
@@ -657,6 +685,18 @@ allocate_handle (void)
 #endif /* HAVE_CURLOPT_PROTOCOLS_STR */
   if (proxy)
     curl_easy_setopt (ch->c, CURLOPT_PROXY, proxy);
+#ifdef HAVE_CURLOPT_PROXY_CAINFO
+  if (proxy_cainfo) {
+    if (strlen (proxy_cainfo) == 0)
+      curl_easy_setopt (ch->c, CURLOPT_PROXY_CAINFO, NULL);
+    else
+      curl_easy_setopt (ch->c, CURLOPT_PROXY_CAINFO, proxy_cainfo);
+  }
+#endif
+#ifdef HAVE_CURLOPT_PROXY_CAPATH
+  if (proxy_capath)
+    curl_easy_setopt (ch->c, CURLOPT_PROXY_CAPATH, proxy_capath);
+#endif
   if (proxy_password)
     curl_easy_setopt (ch->c, CURLOPT_PROXYPASSWORD, proxy_password);
   if (proxy_user)
