@@ -298,6 +298,16 @@ torrent_config_complete (void)
   "upload-rate-limit=BPS          Set upload rate limit (bps)\n" \
   "user-agent=STRING              Set the user-agent"
 
+/* Print some extra information about how the plugin was compiled. */
+static void
+torrent_dump_plugin (void)
+{
+  printf ("libtorrent_variant=rb_libtorrent\n");
+  printf ("libtorrent_version=%s\n", libtorrent::version());
+  printf ("libtorrent_default_user_agent=%s\n",
+          pack.get_str (pack.user_agent).c_str());
+}
+
 /* We got the metadata. */
 static void
 got_metadata (void)
@@ -567,6 +577,31 @@ torrent_pread (void *hv, void *buf, uint32_t count, uint64_t offset,
   return 0;
 }
 
+/* Prefetch data. */
+static int
+torrent_cache (void *hv, uint32_t count, uint64_t offset, uint32_t flags)
+{
+  auto ti = handle.torrent_file();
+
+  while (count > 0) {
+    libtorrent::peer_request part =
+      ti->map_file (index_.load(), offset, (int) count);
+
+    part.length = std::min (ti->piece_size (part.piece) - part.start,
+                            part.length);
+
+    if (! handle.have_piece (part.piece)) {
+      /* Tell the picker that we want this piece sooner. */
+      handle.piece_priority (part.piece, libtorrent::top_priority);
+    }
+
+    count -= part.length;
+    offset += part.length;
+  }
+
+  return 0;
+}
+
 /* https://bugzilla.redhat.com/show_bug.cgi?id=1418328#c9 */
 namespace {
   nbdkit_plugin create_plugin() {
@@ -579,6 +614,7 @@ namespace {
     plugin.config_complete   = torrent_config_complete;
     plugin.config_help       = torrent_config_help;
     plugin.magic_config_key  = "torrent";
+    plugin.dump_plugin       = torrent_dump_plugin,
     plugin.after_fork        = torrent_after_fork;
     plugin.preconnect        = torrent_preconnect;
     plugin.open              = torrent_open;
@@ -586,6 +622,7 @@ namespace {
     plugin.get_size          = torrent_get_size;
     plugin.block_size        = torrent_block_size;
     plugin.pread             = torrent_pread;
+    plugin.cache             = torrent_cache;
     return plugin;
   }
 }
