@@ -46,6 +46,10 @@
 
 #include <pthread.h>
 
+#ifdef HAVE_FNMATCH_H
+#include <fnmatch.h>
+#endif
+
 #define NBDKIT_API_VERSION 2
 #include <nbdkit-plugin.h>
 
@@ -92,6 +96,7 @@ uint16_t create_hwversion =
 uint64_t create_size;                  /* create-size */
 enum VixDiskLibDiskType create_type =
   VIXDISKLIB_DISK_MONOLITHIC_SPARSE;   /* create-type */
+const char *export_wildcard;           /* export */
 const char *filename;                  /* file */
 char *libdir;                          /* libdir */
 uint16_t nfc_host_port;                /* nfchostport */
@@ -233,6 +238,21 @@ vddk_config (const char *key, const char *value)
       return -1;
     }
   }
+  else if (strcmp (key, "export") == 0) {
+#ifdef HAVE_FNMATCH_H
+    if (strcmp (value, "") == 0) {
+      nbdkit_error ("export parameter should be a wildcard, "
+                    "not an empty string");
+      return -1;
+    }
+    export_wildcard = value;
+#else
+    nbdkit_error ("this build of nbdkit does not support the 'export' "
+                  "parameter, because the fnmatch() function is not "
+                  "available in libc");
+    return -1;
+#endif
+  }
   else if (strcmp (key, "file") == 0) {
     /* NB: Don't convert this to an absolute path, because in the
      * remote case this can be a path located on the VMware server.
@@ -318,17 +338,14 @@ vddk_config (const char *key, const char *value)
 static int
 vddk_config_complete (void)
 {
-  if (filename == NULL) {
-    nbdkit_error ("you must supply the file=<FILENAME> parameter "
-                  "after the plugin name on the command line");
+  /* It's never permitted to use both file and export together. */
+  if (filename && export_wildcard) {
+    nbdkit_error ("file and export parameters cannot be used together");
     return -1;
   }
 
-  /* For remote connections, check all the parameters have been
-   * passed.  Note that VDDK will segfault if parameters that it
-   * expects are NULL (and there's no real way to tell what parameters
-   * it is expecting).  This implements the same test that the VDDK
-   * sample program does.
+  /* Is the connection remote?  This implements the same test that the
+   * VDDK sample program does.
    */
   is_remote =
     vmx_spec ||
@@ -340,7 +357,28 @@ vddk_config_complete (void)
     port ||
     nfc_host_port;
 
-  if (is_remote) {
+  if (!is_remote) {
+    /* For local files the 'file' parameter is mandatory.  'export'
+     * cannot be used.  Maybe we can allow it in future.
+     */
+    if (filename == NULL) {
+      nbdkit_error ("you must supply the file=<FILENAME> parameter "
+                    "after the plugin name on the command line");
+      return -1;
+    }
+  }
+  else {
+    /* For remote connections, check all the parameters have been
+     * passed.  Note that VDDK will segfault if parameters that it
+     * expects are NULL (and there's no real way to tell what
+     * parameters it is expecting).
+     */
+    if (!filename && !export_wildcard) {
+      nbdkit_error ("remote connection requested, either file or export "
+                    "parameter is required");
+      return -1;
+    }
+
 #define missing(test, param)                                            \
     if (test) {                                                         \
       nbdkit_error ("remote connection requested, missing parameter: %s", \
@@ -375,14 +413,15 @@ vddk_config_complete (void)
 }
 
 #define vddk_config_help \
-  "[file=]<FILENAME>   (required) The filename (eg. VMDK file) to serve.\n" \
+  "[file=]<FILENAME>              The filename (eg. VMDK file) to serve.\n" \
+  "export=<WILDCARD>              Let client choose file matching wildcard.\n" \
   "Many optional parameters are supported, see nbdkit-vddk-plugin(1)."
 
 static void
 missing_required_symbol (const char *fn)
 {
   nbdkit_error ("required VDDK symbol \"%s\" is missing. "
-                "VDDK version must be >= 6.5. "
+                "VDDK version must be >= 6.7. "
                 "See nbdkit-vddk-plugin(1) man page "
                 "section \"SUPPORTED VERSIONS OF VDDK\". "
                 "Original dlopen error: %s\n",
@@ -401,7 +440,7 @@ load_library (bool load_error_is_fatal)
     /* Prefer the newest library in case multiple exist.  Check two
      * possible directories: the usual VDDK installation puts .so
      * files in an arch-specific subdirectory of $libdir (our minimum
-     * supported version is VDDK 6.5, which only supports x64-64); but
+     * supported version is VDDK 6.7, which only supports x64-64); but
      * our testsuite is easier to write if we point libdir directly to
      * a stub .so.
      */
@@ -668,37 +707,6 @@ vddk_dump_plugin (void)
 /* Lock protecting open/close calls - see above. */
 static pthread_mutex_t open_close_lock = PTHREAD_MUTEX_INITIALIZER;
 
-static inline VixDiskLibConnectParams *
-allocate_connect_params (void)
-{
-  VixDiskLibConnectParams *ret;
-
-  if (VixDiskLib_AllocateConnectParams != NULL) {
-    VDDK_CALL_START (VixDiskLib_AllocateConnectParams, "")
-      ret = VixDiskLib_AllocateConnectParams ();
-    VDDK_CALL_END (VixDiskLib_AllocateConnectParams, 0);
-  }
-  else
-    ret = calloc (1, sizeof (VixDiskLibConnectParams));
-
-  return ret;
-}
-
-static inline void
-free_connect_params (VixDiskLibConnectParams *params)
-{
-  /* Only use FreeConnectParams if AllocateConnectParams was
-   * originally called.  Otherwise use free.
-   */
-  if (VixDiskLib_AllocateConnectParams != NULL) {
-    VDDK_CALL_START (VixDiskLib_FreeConnectParams, "params")
-      VixDiskLib_FreeConnectParams (params);
-    VDDK_CALL_END (VixDiskLib_FreeConnectParams, 0);
-  }
-  else
-    free (params);
-}
-
 /* Create the per-connection handle. */
 static void *
 vddk_open (int readonly)
@@ -715,11 +723,54 @@ vddk_open (int readonly)
     nbdkit_error ("calloc: %m");
     return NULL;
   }
+  h->readonly = readonly;
   h->commands = (command_queue) empty_vector;
   pthread_mutex_init (&h->commands_lock, NULL);
   pthread_cond_init (&h->commands_cond, NULL);
 
-  h->params = allocate_connect_params ();
+  /* Choose the file to open.  If 'export' was passed we use the NBD
+   * export name, but check that it matches the wildcard.
+   */
+  if (!export_wildcard)
+    h->filename = filename;
+  else {
+#ifdef HAVE_FNMATCH_H
+    int r;
+    const char *nbd_export_name = nbdkit_export_name ();
+
+    if (nbd_export_name == NULL) {
+      nbdkit_error ("'export' parameter used, but the client did not "
+                    "negotiate an export name.  Using oldstyle protocol?");
+      goto err0;
+    }
+
+    r = fnmatch (export_wildcard, nbd_export_name, FNM_PATHNAME);
+    switch (r) {
+    case 0: /* OK */ break;
+    case FNM_NOMATCH:
+      nbdkit_error ("access denied: client requested \"%s\" "
+                    "which does not match export=\"%s\"",
+                    nbd_export_name, export_wildcard);
+      goto err0;
+    default:
+      nbdkit_error ("fnmatch returned error code %d: %m", r);
+      goto err0;
+    }
+
+    /* This is OK, it's valid through to .close */
+    h->filename = nbd_export_name;
+#else
+    /* This can never be reached because the parameter won't be parsed
+     * by vddk_config above.
+     */
+    abort ();
+#endif
+  }
+  assert (h->filename != NULL);
+
+  VDDK_CALL_START (VixDiskLib_AllocateConnectParams, "")
+    h->params = VixDiskLib_AllocateConnectParams ();
+  VDDK_CALL_END (VixDiskLib_AllocateConnectParams, 0);
   if (h->params == NULL) {
     nbdkit_error ("allocate VixDiskLibConnectParams: %m");
     goto err0;
@@ -781,13 +832,20 @@ vddk_open (int readonly)
       .physicalSectorSize = 0
     };
 
+    /* NBD export names can never be used when creating files.  Assert
+     * this here to make sure.
+     */
+    assert (filename != NULL);
+    assert (strcmp (filename, h->filename) == 0);
+
     VDDK_CALL_START (VixDiskLib_Create,
                      "h->connection, %s, &cparams, NULL, NULL",
-                     filename)
-      err = VixDiskLib_Create (h->connection, filename, &cparams, NULL, NULL);
+                     h->filename)
+      err = VixDiskLib_Create (h->connection, h->filename, &cparams,
+                               NULL, NULL);
     VDDK_CALL_END (VixDiskLib_Create, 0);
     if (err != VIX_OK) {
-      VDDK_ERROR (err, "VixDiskLib_Create: %s", filename);
+      VDDK_ERROR (err, "VixDiskLib_Create: %s", h->filename);
       goto err2;
     }
 
@@ -809,11 +867,11 @@ vddk_open (int readonly)
   }
 
   VDDK_CALL_START (VixDiskLib_Open,
-                   "connection, %s, %d, &handle", filename, flags)
-    err = VixDiskLib_Open (h->connection, filename, flags, &h->handle);
+                   "connection, %s, %d, &handle", h->filename, flags)
+    err = VixDiskLib_Open (h->connection, h->filename, flags, &h->handle);
   VDDK_CALL_END (VixDiskLib_Open, 0);
   if (err != VIX_OK) {
-    VDDK_ERROR (err, "VixDiskLib_Open: %s", filename);
+    VDDK_ERROR (err, "VixDiskLib_Open: %s", h->filename);
 
     /* Attempt to advise the user on the extremely helpful "Unknown
      * error" result of VixDiskLib_Open().
@@ -852,7 +910,9 @@ vddk_open (int readonly)
     VixDiskLib_Disconnect (h->connection);
   VDDK_CALL_END (VixDiskLib_Disconnect, 0);
  err1:
-  free_connect_params (h->params);
+  VDDK_CALL_START (VixDiskLib_FreeConnectParams, "params")
+    VixDiskLib_FreeConnectParams (h->params);
+  VDDK_CALL_END (VixDiskLib_FreeConnectParams, 0);
  err0:
   pthread_mutex_destroy (&h->commands_lock);
   pthread_cond_destroy (&h->commands_cond);
@@ -871,6 +931,8 @@ vddk_close (void *handle)
   send_command_and_wait (h, &stop_cmd);
   pthread_join (h->thread, NULL);
 
+  nbdkit_extents_free (h->extents);
+
   VDDK_CALL_START (VixDiskLib_Close, "handle")
     VixDiskLib_Close (h->handle);
   VDDK_CALL_END (VixDiskLib_Close, 0);
@@ -878,7 +940,9 @@ vddk_close (void *handle)
     VixDiskLib_Disconnect (h->connection);
   VDDK_CALL_END (VixDiskLib_Disconnect, 0);
 
-  free_connect_params (h->params);
+  VDDK_CALL_START (VixDiskLib_FreeConnectParams, "params")
+    VixDiskLib_FreeConnectParams (h->params);
+  VDDK_CALL_END (VixDiskLib_FreeConnectParams, 0);
   pthread_mutex_destroy (&h->commands_lock);
   pthread_cond_destroy (&h->commands_cond);
   command_queue_reset (&h->commands);
@@ -944,7 +1008,7 @@ vddk_block_size (void *handle,
   return 0;
 }
 
-/* The Flush call was added in VDDK 6.0, since we support minimum 6.5
+/* The Flush call was added in VDDK 6.0, since we support minimum 6.7
  * we are always able to do FUA / flush.
  */
 static int
