@@ -464,8 +464,10 @@ nbdkit_parse_delay (const char *what, const char *str,
 
   if (rsec)
     *rsec = d;
-  if (rnsec)
-    *rnsec = (d - *rsec) * 1000000000.;
+  if (rnsec) {
+    long m = lrint ((d - *rsec) * 1000000000.);
+    *rnsec = m;
+  }
 
   return 0;
 }
@@ -1073,7 +1075,7 @@ nbdkit_peer_security_context (void)
   }
   else if (r == -1 && errno == ENOPROTOOPT) {
     /* This is not really an error so don't call nbdkit_error. */
-    nbdkit_debug ("getsockopt: SO_PEERSEC: %m");
+    debug ("getsockopt: SO_PEERSEC: %m");
     return NULL;
   }
   else if (r == -1 && errno != ERANGE) {
