@@ -30,8 +30,10 @@
 # OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 # SUCH DAMAGE.
 
-set -e
 source ./functions.sh
+set -e
+set -x
+set -u
 
 output="$(nbdkit example4 --dump-plugin)"
 if [[ ! ( "$output" =~ example4_extra\=hello ) ]]; then
