@@ -41,6 +41,8 @@
 
 source ./functions.sh
 set -e
+set -x
+set -u
 
 requires_run
 requires_plugin sh
