@@ -33,23 +33,24 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 # Test informational error messages sent to the NBD client.
 # qemu-img supports this since at least 2.12.0.
 
 requires_run
 requires_plugin eval
-requires "$QEMU_IMG" --version
+requires qemu-img --version
 
 out=last-error.out
 rm -f $out
 cleanup_fn rm -f $out
 
-export out QEMU_IMG
+export out
 
 nbdkit eval \
        open=' echo EPERM Go Away >&2; exit 1 ' get_size=' echo 0 ' \
-       --run ' "$QEMU_IMG" info "$uri" > $out 2>&1 ||: '
+       --run ' qemu-img info "$uri" > $out 2>&1 ||: '
 cat $out
 
 grep "Go Away" $out
