@@ -33,13 +33,13 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_nbdcopy
 requires nbdsh --version
 requires_nbdsh_uri
-requires "$QEMU_IMG" --version
-export QEMU_IMG
-requires bash -c '"$QEMU_IMG" --help | grep -- --target-image-opts'
+requires qemu-img --version
+requires bash -c 'qemu-img --help | grep -- --target-image-opts'
 requires hexdump --version
 requires $TRUNCATE --version
 requires_filter luks
@@ -66,12 +66,12 @@ rm -f $encrypt_disk $plain_disk $pid $sock
 # starts with random plaintext.
 #
 # https://stackoverflow.com/a/44669936
-"$QEMU_IMG" create -f luks \
+qemu-img create -f luks \
          --object secret,data=123456,id=sec0 \
          -o key-secret=sec0 \
          $encrypt_disk 1M
 $TRUNCATE -s 1M $plain_disk
-"$QEMU_IMG" convert --target-image-opts -n \
+qemu-img convert --target-image-opts -n \
          --object secret,data=123456,id=sec0 \
          $plain_disk \
          driver=luks,file.filename=$encrypt_disk,key-secret=sec0
@@ -110,7 +110,7 @@ nbdsh -u "$uri" \
 
 # Use qemu to copy out the whole disk.  Note we called flush() above
 # so the disk should be synchronised.
-"$QEMU_IMG" convert --image-opts \
+qemu-img convert --image-opts \
          --object secret,data=123456,id=sec0 \
          driver=luks,file.filename=$encrypt_disk,key-secret=sec0 \
          $plain_disk
