@@ -33,12 +33,13 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_run
 requires test -f disk
 requires_nbdcopy
 requires_nbdinfo
-requires "$QEMU_IMG" --version
+requires qemu-img --version
 requires cmp --version
 
 # Check nbdinfo supports the --is flag (not RHEL 8).
@@ -53,8 +54,8 @@ rm -f $files
 cleanup_fn rm -f $files
 
 # Create a qcow2 disk for testing.
-"$QEMU_IMG" convert -f raw disk -O qcow2 $qcow2 -c
-#"$QEMU_IMG" convert -f raw disk -O qcow2 $qcow2 -c -o compression_type=zstd
+qemu-img convert -f raw disk -O qcow2 $qcow2 -c
+#qemu-img convert -f raw disk -O qcow2 $qcow2 -c -o compression_type=zstd
 
 # Start nbdkit.
 start_nbdkit -P $pid -U $sock \
