@@ -33,6 +33,7 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_run
 requires_plugin sh
@@ -48,10 +49,7 @@ echo 0 > retry-open2-count
 start_t=$SECONDS
 
 # Create a custom plugin which will test retrying open.
-nbdkit -v \
-       sh - \
-       --filter=retry retry-delay=1 \
-       --run 'qemu-io -f raw -c "r -P0 0 512" "$uri" || :' <<'EOF'
+define plugin <<'EOF'
 #!/usr/bin/env bash
 case "$1" in
     open)
@@ -68,6 +66,11 @@ case "$1" in
 esac
 EOF
 
+nbdkit -v \
+       sh - <<<"$plugin" \
+       --filter=retry retry-delay=1 \
+       --run 'qemu-io -f raw -c "r -P0 0 512" "$uri" || :'
+
 # In this test we should see 1 failure:
 # first open FAILS
 # retry and wait 1 second
