@@ -33,6 +33,7 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_run
 requires_plugin sh
@@ -47,10 +48,7 @@ touch retry-request-count
 start_t=$SECONDS
 
 # Create a custom plugin which will test retrying requests.
-nbdkit -v \
-       sh - \
-       --filter=retry-request retry-request-retries=3 retry-request-delay=1 \
-       --run 'nbdcopy --synchronous "$uri" retry-request.img' <<'EOF'
+define plugin <<'EOF'
 #!/usr/bin/env bash
 case "$1" in
     get_size) echo 512 ;;
@@ -71,6 +69,11 @@ case "$1" in
 esac
 EOF
 
+nbdkit -v \
+       sh - <<<"$plugin" \
+       --filter=retry-request retry-request-retries=3 retry-request-delay=1 \
+       --run 'nbdcopy --synchronous "$uri" retry-request.img'
+
 # In this test we should see 3 failures:
 # pread FAILS
 # retry and wait 1 second
