@@ -36,12 +36,13 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_run
 requires test -f disk
 requires tar --version
-requires "$QEMU_IMG" --version
-requires "$QEMU_IMG" info --output=json /dev/null
+requires qemu-img --version
+requires qemu-img info --output=json /dev/null
 requires_nbdcopy
 requires jq --version
 requires $STAT --version
@@ -56,10 +57,8 @@ files="$disk $raw $out $tar $tar_xz"
 rm -f $files
 cleanup_fn rm -f $files
 
-export QEMU_IMG
-
 # Create a tar file containing a known qcow2 file.
-"$QEMU_IMG" convert -f raw disk -O qcow2 $disk
+qemu-img convert -f raw disk -O qcow2 $disk
 tar cf $tar $disk
 xz --best --block-size=32768 $tar
 
@@ -68,7 +67,7 @@ nbdkit file $tar_xz \
        --filter=qcow2dec \
        --filter=tar tar-entry=$disk \
        --filter=xz \
-       --run '"$QEMU_IMG" info --output=json "$uri"' > $out
+       --run 'qemu-img info --output=json "$uri"' > $out
 cat $out
 
 # Check various fields in the input.
