@@ -33,8 +33,9 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
-requires test "x$vddkdir" != "x"
+requires test "${vddkdir:-}" != ""
 requires test -d "$vddkdir"
 requires test -f "$vddkdir/lib64/libvixDiskLib.so"
 requires $CUT --version
